#picaxe 14m2

symbol PWM_PIN = C.0
symbol SPI_SDO = B.5 ; bei 14M2
symbol SPI_SCK = B.3 ; bei 14M2
symbol CS_SPI_RAM = C.4

symbol SPI_RAM_SIZE_KB = 128
symbol NUMBER_OF_LEDS = 40
symbol LED_BRIGHTNESS_PERCENT = 25

symbol LAST_LED = NUMBER_OF_LEDS - 1
symbol TOTAL_BITS_LEDS = NUMBER_OF_LEDS * 24
symbol TRANSFER_TIME_LEDS = 48 * NUMBER_OF_LEDS ; time in microseconds to transfer all LED data: 2us/bit * 24bit
symbol LED_UPDATE_TIME_MIN_MS = TRANSFER_TIME_LEDS / 125 ; minimum time in milliseconds to update all LEDs @ 32 MHz processor clock (*8/1000)
symbol LED_UPDATE_TIME = LED_UPDATE_TIME_MIN_MS + 8 ; time in milliseconds to update all LEDs

symbol LED_PATTERN_0 = 0x40
symbol LED_PATTERN_1 = 0x78

symbol SPI_RAM_WRITE = 0x02
symbol SPI_RAM_READ  = 0x03

symbol SSP1CON1_SFR = 0x95
symbol SSP1STAT_SFR = 0x94
symbol SSP1BUF_SFR  = 0x91

symbol byte0 = b0 ; muss b0 sein muss wegen Bitzugriff
symbol color = b1
symbol addr_lw = w1
symbol addr_7_0 = b2
symbol addr_15_8 = b3
symbol addr_hw = w2
symbol addr_23_16 = b4
symbol spi_ram_size = b6
symbol cnt1 = b7
symbol color_green = b8
symbol color_red = b9
symbol color_blue = b10
symbol showNumber = w6
symbol cnt_w = w7
symbol value_w = w8
symbol patternNumber = w9
symbol ledNumber = w10
symbol tempWord = w11
symbol pauseTime = w12


#macro PAUSE_MS(x)
   pauseTime = x*8
   pause pauseTime
#endmacro

#macro DEFINE_PATTERN_NUMBER(par_patternnumber)
   patternNumber = par_patternnumber
#endmacro

#macro DEFINE_LED_COLOR(r,g,b)
   color_red = r
   color_green = g
   color_blue = b
#endmacro

#macro SELECT_COLOR(x)
   color = x
   gosub selectColor
#endmacro

#macro LED_PATTERN_SET_LED(par_lednumber)
   ledNumber = par_lednumber
   gosub ledPattern_setLed
#endmacro

#macro LED_PATTERN_SET_NEXT_LED
   gosub ledPattern_setNextLed
#endmacro

#macro LED_PATTERN_FILL
   gosub ledPattern_fill
#endmacro

#macro LED_PATTERN_END
  gosub ledPattern_end
#endmacro

#macro SHOW_LEDS(par_shownumber)
   showNumber = par_shownumber
   gosub showLeds
#endmacro

   

main:
setfreq m32
spi_ram_size = SPI_RAM_SIZE_KB

gosub setup

prog_loop:

   DEFINE_PATTERN_NUMBER(0)
   DEFINE_LED_COLOR(0,0,0)
   LED_PATTERN_FILL

   for cnt1 = 1 to 8
      SELECT_COLOR(cnt1)
      for cnt_w = 0 to 19
         LED_PATTERN_SET_LED(cnt_w)
         LED_PATTERN_SET_LED(LAST_LED - cnt_w)
         SHOW_LEDS(0)
         PAUSE_MS(10)
      next cnt_w
      PAUSE_MS(500)
   next cnt1
   PAUSE_MS(500)

   for cnt1 = 1 to 7
      SELECT_COLOR(cnt1)
      LED_PATTERN_FILL
      SHOW_LEDS(0)
      PAUSE_MS(500)
   next cnt1
   PAUSE_MS(500)
   
   SHOW_LEDS(1)
   PAUSE_MS(2000)

goto prog_loop


setup:
   high CS_SPI_RAM
   gosub initSpi
   gosub zero_spiram

   DEFINE_PATTERN_NUMBER(0)
   DEFINE_LED_COLOR(0,0,0)
   LED_PATTERN_FILL

   DEFINE_PATTERN_NUMBER(1)
   cnt1 = 1
   value_w = NUMBER_OF_LEDS / 2 - 1
   for cnt_w = 0 to value_w
      SELECT_COLOR(cnt1)
      LED_PATTERN_SET_LED(cnt_w)
      LED_PATTERN_SET_LED(LAST_LED - cnt_w)
      inc cnt1
      if cnt1 = 7 then
         cnt1 = 1
      endif
   next cnt_w
   LED_PATTERN_END
return


initSpi:
   low SPI_SCK  ; SCK als Ausgang low
   low SPI_SDO  ; SDO als Ausgang low
   pokesfr SSP1CON1_SFR, 0x22 ; SSPEN=1, CKP=0, SSPM=0010 (500kHz)
   pokesfr SSP1STAT_SFR, 0x40 ; SMP=0, CKE=1
return


updateLeds: ; Parameter: showNumber
  addr_lw = showNumber * TOTAL_BITS_LEDS;
  addr_hw = showNumber ** TOTAL_BITS_LEDS;
  high CS_SPI_RAM
  gosub initSpi
  low CS_SPI_RAM
  pokesfr SSP1BUF_SFR, SPI_RAM_READ
  if spi_ram_size > 64 then
     pokesfr SSP1BUF_SFR, addr_23_16 ; addr 23..16
  endif
  pokesfr SSP1BUF_SFR, addr_15_8 ; addr 15..8
  pokesfr SSP1BUF_SFR, addr_7_0 ; addr 7..0

  input SPI_SCK
  low PWM_PIN
  pwmout PWM_PIN, 1, 3 ; 4 MHz PWM @ 32 MHz PICAXE clock
return


terminateDataToLeds:
  high CS_SPI_RAM
  pwmout PWM_PIN, off
  input PWM_PIN
  low SPI_SCK
return


showLeds: ; Parameter: showNumber
   gosub updateLeds
   pause LED_UPDATE_TIME
   gosub terminateDataToLeds
return


spiSendColor: ; Parameter: byte0 contains color
   if bit7 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit6 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit5 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit4 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit3 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit2 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit1 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
   if bit0 = 1 then
      pokesfr SSP1BUF_SFR, LED_PATTERN_1
   else
      pokesfr SSP1BUF_SFR, LED_PATTERN_0
   endif
return


zero_spiram:
  gosub initSpi
  low CS_SPI_RAM
  pokesfr SSP1BUF_SFR, SPI_RAM_WRITE
  if spi_ram_size > 64 then
     pokesfr SSP1BUF_SFR, 0 ; addr 23..16
  endif
  pokesfr SSP1BUF_SFR, 0 ; addr 15..8
  pokesfr SSP1BUF_SFR, 0 ; addr 7..0

  input SPI_SCK
  low PWM_PIN
  pwmout PWM_PIN, 1, 3 ; 4 MHz PWM @ 32 MHz PICAXE clock
  pause 2400 ; 300ms
  high CS_SPI_RAM
  pwmout PWM_PIN, off
  input PWM_PIN
  low SPI_SCK
return


ledPattern_begin: ; Parameter: patternNumber, ledNumber 
  high CS_SPI_RAM
  gosub initSpi
  low CS_SPI_RAM
  pokesfr SSP1BUF_SFR, SPI_RAM_WRITE
  tempWord = patternNumber * NUMBER_OF_LEDS + ledNumber
  addr_lw = tempWord * 24;
  addr_hw = tempWord ** 24;
  if spi_ram_size > 64 then
     pokesfr SSP1BUF_SFR, addr_23_16 ; addr 23..16
  endif
  pokesfr SSP1BUF_SFR, addr_15_8 ; addr 15..8
  pokesfr SSP1BUF_SFR, addr_7_0 ; addr 7..0
return


ledPattern_setLed: ; Parameter: patternNumber, ledNumber
  gosub ledPattern_begin
  gosub ledPattern_setNextLed
return


ledPattern_setNextLed:
  byte0 = color_green * LED_BRIGHTNESS_PERCENT / 100
  gosub spiSendColor
  byte0 = color_red * LED_BRIGHTNESS_PERCENT / 100
  gosub spiSendColor
  byte0 = color_blue * LED_BRIGHTNESS_PERCENT / 100
  gosub spiSendColor
return


ledPattern_fill:
   ledNumber = 0
   gosub ledPattern_begin
   for tempWord = 0 to LAST_LED
      LED_PATTERN_SET_NEXT_LED
   next tempWord
return


ledPattern_end:
  high CS_SPI_RAM
return


selectColor: ; Parameter: color
   select case color
     case 0,8
        DEFINE_LED_COLOR(0,0,0)
     case 1
        DEFINE_LED_COLOR(0xFF,0,0)
     case 2
        DEFINE_LED_COLOR(0xFF,0xFF,0)
     case 3
        DEFINE_LED_COLOR(0,0xFF,0)
     case 4
        DEFINE_LED_COLOR(0,0xFF,0xFF)
     case 5
        DEFINE_LED_COLOR(0,0,0xFF)
     case 6
        DEFINE_LED_COLOR(0xFF,0,0xFF)
     case 7
        DEFINE_LED_COLOR(0xFF,0xFF,0xFF)
   endselect
return





